/* M. Angela Osterman
   June 20, 2003 
   This program is designed to reproduce the power-source size tracks of
    Blundell et al. The projected size is calculated in another program.

   Paramita Barai
   Inputting correction in frequency caused by z so let source evolve in 
   power at nu = nu*(1+z) = Rest frame frequency emitted
   
 (Assuming that have gotten Blundell's theoretical P-D curves)
 Modifying to do the big Monte Carlo simulation of random sources ..

blunflim.c

2004:
Aug 18: Incorporating Projection Effect into Length / Size of radio source
Using both methods now: BRW - random angle, KDA - fixed sngle

Sept 03: Follow BRW method of getting Projected Size -- as much more reasonable
Code running w/ different value of power law index (x = 3).

Nov 06: (1st run after Prospectus) Modify to rest frame. Run w/ pm = 2.01. Write P, z in separate file than D

2005:
Mar 21: Final version of BRW code, keeping a copy here to be used by any variaiton of BRW case
Aug 02: Modify for combining this big run along with all later small runs.
        Read input model parameters from separate file.
Aug 03: Decreasing ntest to ntest = 5000, (from ntest = 40000) to 
	GREATLY improve on BRW execution speed.
	NO Difference in Resulting P_151 when seen by plotting in gnuplot.
	Consistent upto at least 3-4 sig figs.		
Sept 05: To improve on speed of execution of CASE 4 change
         dt = (tobs-tmin_sec)/1.e5;   
      TO dt = (tobs-tmin_sec)/1.e4;
NO difference in P_151 when plotted in gnuplot, or seen from actual values till 6 decimal places
Sept 19: Read SIZ from external file 

Oct 01: Vary r_hs with D (i.e. with t)

Case2 -- Single Quadratic (2nd order) fit of r_hs vs. D data 
*/
// NOTE; in KDA and MK, tinj is not constant. should that be the case
//   here too?

#include <math.h>
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include "nrutil.h"   // make sure this header file is in this directory.
#include "nr.h"

// the following are required for the Numerical Recipes functions. 
#define EPS 1.0e-6
#define JMAX 20
#define JMAXP (JMAX+1) 
#define Kconst 5
#define NRANSI
#define FUNC(x) ((*func)(x))
#define NR_END 1
#define FREE_ARG char*

// This block contains universal constants.
#define c 2.9979e8            // speed of light in m/s.
#define sigmae 6.652e-29      // Thompson cross section in m^2.
#define me 9.1094e-31         // electron mass in kg.
#define mu0 4.0*M_PI*1.0e-7   // permeability of free space in N*A^-2.
#define secpermyr 3.15576e13  // the number of seconds in a Myr.
#define mperkpc 3.085678e19   // the number of m in a kpc.
#define a 7.566e-16           // radiation density constant in J*m^-3*K^-4
#define e 4.803e-10           // charge of an electron in e.s.u.
#define ecoul 1.601e-19       // charge of an electron in coulombs

// These factors are declared globally because many functions will use them.
double betaa, a0, rho0, c1, pm, gammin, gammax, Q0, tobs, tinj;
double ucmb, gaml, gamx, nu, nu0, Pconst, a3, a4, a1;
double tbs_sec, tbf_sec, tmin_sec, r;
double c7, tbs, tbf, Bcmb;
double r0, F0, yexp1, yexp2;

int kmax, kount;
double *xp, **yp, dxsav;
double eps, h1, hmin, k1, k2, *fgidist;

int main()
{
void P1term1(double , double [], double []);
void P2term1(double , double [], double []);
void odeint(double ystart[], int nvar, double x1, double x2, double eps, double h1,
        double hmin, int *nok, int *nbad,
        void (*derivs)(double, double [], double []),
        void (*rkqs)(double [], double [], int, double *, double, double, double [],
        double *, double *, void (*)(double, double [], double [])));

int nok, nbad;
double tqzvals[6];
double z,  tobs0, a0kpc, siz_hs; 
double ntestd, dtest, test, ta13, dt, gamtest, tgambs, ttest;
double P1, P2, P3;
int ni, n, nm, m;
long ntest, j, case1, case2, case4, SIZ;
FILE *stream1, *stream2, *fopen();
double D(), pl(), uB(), ue(), nug(), a2(), gamma(), rhs(), Ahs(), Bhs(), gam0(), gambs(), gambf(), f(), nb();

  xp = dvector(1, 200);
  yp = dmatrix(1, 10, 1, 200);
  fgidist = dvector(1, 1);

printf(" Within C Code 'blunflim.c' for BIG RUN with initial population \n\n");

  stream1 = fopen("../SampSize3C6C7C.dat","r");
  if ( (stream1 = fopen("../SampSize3C6C7C.dat","r")) == NULL)
      {   printf("Can't open %s \n","../SampSize3C6C7C.dat");   exit(1);   }
  fscanf(stream1,"%u",&SIZ);
  fclose(stream1);
  printf(" SIZ = %u\n", SIZ);	

  stream1 = fopen("../BRWparams.dat","r");
  if ( (stream1 = fopen("../BRWparams.dat","r")) == NULL)
      {   printf("Can't open %s \n","../BRWparams.dat");   exit(1);   }

  fscanf(stream1,"%lg",&betaa);
  fscanf(stream1,"%lg",&a0kpc);
  fscanf(stream1,"%lg",&rho0);
  fscanf(stream1,"%lg",&gamx);
  fscanf(stream1,"%lg",&gaml);
  fscanf(stream1,"%lg",&gammin);
  fscanf(stream1,"%lg",&gammax);
  fscanf(stream1,"%lg",&pm);
//  fscanf(stream1,"%lg",&rhs);
  fscanf(stream1,"%lg",&r0);
  fscanf(stream1,"%lg",&tbs);
  fscanf(stream1,"%lg",&tbf);
  fscanf(stream1,"%lg",&nu0);
  fclose(stream1);

  // Constants obtained from Fitting Jeyakumar & Saikia, 2000 Data of D_hs vs. D_proj.
// Case 2 : Single Quadratic Fit (2nd Order)
// For Case 2 : r0 = 0.21671769,  F0 = 0.020396732,  y1 = 1.0526278,  y2 = -0.030605459

  r0 = 0.21671769;
  F0 = 0.020396732;
  yexp1 = 1.0526278;
  yexp2 = -0.030605459;

  a0 = a0kpc*mperkpc;     // factor a0. Blundell et al. use 10kpc, converted here to meters.
//  Ahs = M_PI*pow(rhs, 2.0)*mperkpc*mperkpc;    // hotspot area in m^2, Blundell p693
  c1 = 1.8;              // factor c1, dimensionless, Blundell p690
  c7 = 1.12e18;          // factor from Blundell(Leahy) in nT^3*year^2*Mhz, p692
// note difference in unit used here & that mentioned in paper,
// 10^3 nT^3*Myr^2*GHz = 10^18 nT^3*year^2*Mhz, so nu_bh obtained from this c7 will be in MHz

  tbs_sec = tbs*3.15576e7;     // converts tbs from years to seconds
  tbf_sec = tbf*3.15576e7;     // converts tbf from years to seconds

  // EDIT NAMES OF MAJOR INPUT & OUTPUT FILES HERE. 
  // open file containing t,Q0,z data.
  stream1 = fopen("tbirztageQ.dat","r");
  if ( (stream1 = fopen("tbirztageQ.dat","r")) == NULL)
       {   printf("Can't open %s \n","tbirztageQ.dat");   exit(1);   }
  // open file to save z, D, P data.
  stream2 = fopen("logP151_1pz.dat","w");
  if ( (stream2 = fopen("logP151_1pz.dat","w")) == NULL)
       {   printf("Can't open %s \n","logP151_1pz.dat");   exit(1);   }

case1 = 0;
case2 = 0;
case4 = 0;
for (j = 0 ; j < SIZ ; j++)
 {
  fscanf(stream1,"%lg",&tqzvals[0]);    // gets tbir
  fscanf(stream1,"%lg",&tqzvals[1]);    // gets z
  fscanf(stream1,"%lg",&tqzvals[2]);    // gets tage
  fscanf(stream1,"%lg",&tqzvals[3]);    // gets Q

  // This block contains the main independent variables of the simulation.
  // We will want to step through values of t (observed).
  z = tqzvals[1];                  // redshift, variable
  nu = nu0*(1+z);                  // changing frequency to rest frame frequency emitted
  
  tobs = tqzvals[2] * secpermyr;   // age of source when observed in seconds
  Q0 = tqzvals[3];                 // initial energy of source in Watts.

//  printf("z = %e\t tobs = %e Myr\t Q0 = %e \n", z, tobs/secpermyr, Q0);

  // This block contain parameters which depend on values defined above
  //   and do not change with time.
  ucmb = a*pow(2.74,4.0)*pow((1.0 + z),4.0);  // CMB energy density in J*m^-3
  Bcmb = 0.325*pow((1.0 + z),2.0);            // B field of CMB in nT, Blundell p678
  r = (1.0 + pm)/4.0;                         // ratio of B energy density uB to the sum 
// of the energy densities of the particles (ue + uT), different for different regions of pm, see fig 11, p694
                                                                                                
  a1 = (4.0 + betaa)/(gaml*(5.0 - betaa));    // a1, a3, a4 are factors from Kaiser
                                              // Kaiser gamc = Blundell gaml
  a3 = 1.0 - a1*(gaml + 1.0/3.0);             //defined below Kaiser eq. 7; no gamB in Blundell
                                              // as values same(=4/3) use gaml in place of gam
  a4 = 1.0 - a1/3.0;
 // Bhs = sqrt(3.0*mu0*Q0/(c*Ahs)) * 1.0e9;    // B field of hotspot in nT,  Blundell p692, eq 11
 // gam0 = sqrt(me*2.0*M_PI*nu/ecoul/(Bhs/1.0e9));
 // printf("gam0 = %e \n", gam0); 
 // Lorentz factor of a relativistic, radiating electron. BRW eq 3, p687

//  nubs = (9.0/4.0)*c7*Bhs/((Bhs*Bhs + Bcmb*Bcmb)*(Bhs*Bhs + Bcmb*Bcmb)*(tbs*tbs));
//  nubf = (9.0/4.0)*c7*Bhs/((Bhs*Bhs + Bcmb*Bcmb)*(Bhs*Bhs + Bcmb*Bcmb)*(tbf*tbf));
//  printf("\n nubs = %e, nubf = %e \n", nubs, nubf);
     // slow & fast break freq. (in MHz) corresponding to tbs and tbf, Blundell eq. 12
     // NOTE still unclear as to the "ts" in the paper.

 // Pconst = (1.0/6.0/M_PI/nu)*sigmae*c*Q0*pow( (Q0/(c*Ahs)),((1.0 - gaml)/gaml) );
     // const at beginning of power eqn mult. by constants w/in eqn(Blun21)
  Pconst = (1.0/6.0/M_PI/nu)*sigmae*c*Q0*pow( (Q0/c),((1.0 - gaml)/gaml) );

  // FOLLOWING PART TAKEN FROM KAISER CODE, TO FIND tmin
  // determine the minimum value of ti for a given t
  // this comes from the lowest ti such that gamma_i is finite
  ntest = 5000;
  ntestd = 5000.0;      // ntest = integer version of ntestd.
  dtest = tobs/ntestd;
  test = tobs ;
  ta13 = pow(tobs,(-a1/3.0));
  for (n=1 ; n<=ntest ; n++)
    {
      test = test - dtest;
      if ( ta13 < a2(test)*gamma(test) ) break; 
    }
  tmin_sec = test + dtest;
//printf("tmin = %e Myr, tbs=%e Yr, tbf=%e Yr, tobs = %e Myr \n",tmin_sec/secpermyr, tbs, tbf, tobs/secpermyr );
//printf("gami(tmin) = %e, gambs(tmin)=%e, gambf(tmin)=%e \n",gamma(tmin_sec), gambs(tmin_sec), gambf(tmin_sec));

// The next block of code integates for power if case 1 in fig 12
   if ( (gamma(tmin_sec) < gambs(tmin_sec)) && (gamma(tobs) < gambs(tobs)) )
   {
//      printf("CASE 1\n");
//      fprintf(stream3," run %d\t CASE 1\n", j );
      k1 = tmin_sec;
      k2 = tobs;
      eps = 1.0e-5;
      h1 = 1.0e-2;
      hmin = 0.0;
      fgidist[1] = 0.0;
      odeint(fgidist, 1, k1, k2, eps, h1, hmin, &nok, &nbad, P1term1, rkqs);
      P1 = fgidist[1];
      P2 = 0;
      P3 = 0;
      case1 = case1 + 1;
   }

// The next block of code integates for power if case 2 in fig 12
   if ( (gamma(tmin_sec)>gambs(tmin_sec)) && (gamma(tmin_sec)<gambf(tmin_sec)) && (gamma(tobs)>gambs(tobs)) && (gamma(tobs)<gambf(tobs)) )
   {
//      printf("CASE 2\n");
//      fprintf(stream3," run %d\t CASE 2\n", j );
      k1 = tmin_sec;
      k2 = tobs;
      eps = 1.0e-5;
      h1 = 1.0e-2;
      hmin = 0.0;
      fgidist[1] = 0.0;
      odeint(fgidist, 1, k1, k2, eps, h1, hmin, &nok, &nbad, P2term1, rkqs);
      P1 = 0;
      P2 = fgidist[1];
      P3 = 0;
      case2 = case2 + 1;
   }


// The next block of code integates for power if case 4 in fig 12
  if ( (gamma(tmin_sec)>gambs(tmin_sec)) && (gamma(tmin_sec)<gambf(tmin_sec)) && (gamma(tobs)<gambs(tobs)) )
  {
//   printf("CASE 4\n");
//   fprintf(stream3," run %d\t CASE 4\n", j );
   nm = 100000;
 // dt = (tobs-tmin_sec)/1.e5;
   dt = (tobs-tmin_sec)/1.e4;
   ttest = tmin_sec;
   for (m=0; m<= nm; m++)
   { 
     gamtest = gamma(ttest);
     if (fabs(gamtest-gambs(ttest)) < 10.0) { break; } 
     ttest = ttest + dt;
   }
   tgambs = ttest;
// printf("m= %d, tgambs = %e Myr, on run=%d \n", m, tgambs/secpermyr, j);
// fprintf(stream5, "m= %d, tgambs = %e Myr \n", m, tgambs/secpermyr);

  // The next block of code integrates P2term
      k1 = tmin_sec;
      k2 = tgambs;
      eps = 1.0e-5;
      h1 = 1.0e-2;
      hmin = 0.0;
      fgidist[1] = 0.0;
      odeint(fgidist, 1, k1, k2, eps, h1, hmin, &nok, &nbad, P2term1, rkqs);
      P2 = fgidist[1];

   // The next block of code integrates P1term
      k1 = tgambs;
      k2 = tobs;
      eps = 1.0e-5;
      h1 = 1.0e-2;
      hmin = 0.0;
      fgidist[1] = 0.0;
      odeint(fgidist, 1, k1, k2, eps, h1, hmin, &nok, &nbad, P1term1, rkqs);
      P1 = fgidist[1];
   
  // fprintf(stream2,"Ending time is %e which should = %e \n",init_tinj,tobs);
  // printf("P1 successfully integrated.\n");
   P3=0;
   case4 = case4 + 1;
  }
// printf(" ON run %d\n", j);
// printf("z = %e\t  logPnu = %e\n", z,  log10(P1 + P2 + P3));
fprintf(stream2," %e\n", log10(P1 + P2 + P3) );
}

printf(" In BIG RUN calculating Powers of sources in primordial population, CASES obtained: \n") ;
printf("case1=%ld,\t case2=%ld,\t case4=%ld,\t Total=%ld\n\n", case1, case2, case4, case1+case2+case4);

fclose(stream1);   // close data file read
fclose(stream2);   // close data file written
// fclose(stream3);

return 0;
}

// ***************************************************************************************

double D(double tt)
{
 return 2.0*c1*a0*pow((Q0 * pow(tt,3.0) / (pow(a0,5.0)*rho0)),(1.0/(5.0-betaa)));
  // source size in meters as f(tobs,Q0), Blundell eq. 6
} 

// ***************************************************************************************

double pl(double t)
{
   return (1.0/6.0)*18.0 / (gamx + 1.0) * rho0 / pow((5.0 - betaa),2.0)
     * pow(c1,(2.0 - betaa)) * a0*a0 * pow(  ( Q0/pow(a0,5.0)/rho0 ),
     ((2.0 - betaa)/(5.0 - betaa))  ) * pow( t,((-4.0 - betaa)/(5.0 - betaa)) );
  // lobe pressure as f(t,Q0), factor of six smaller than the head pressure,
  //     Blundell eq. 9, p691
}

// ***************************************************************************************

double uB(double t)
{
    return r * pl(t) / (gaml - 1.0) / (r + 1.0);   
   // B field energy density, KDA eq. 15 (lower equation)
   // Kaiser's pc = Blundell's pl, Kaiser's gammac = Blundell's gaml
   // only r changes in uB1, uB2, uB3 as r = (1+pm)/4, 
   // & pm (injection index) is different in different regions of Blundell
}

// ***************************************************************************************

double ue(double t)
{
   return uB(t)/r;
   // electron energy density, KDA eq. 15 (upper equation)
   // Kaiser k' = 0
}

// ***************************************************************************************

double nug(double t)
{  
  // Blundell p693, after eq. 13
  // nonrelativistic cyclotron freq. in sec^-1: Binj obtained in Tesla from the 1st eqn
  // charge of electron written in Coul, & mass of electron in kg 
  double Binj;
  Binj = sqrt(uB(t)*2*mu0);
  return (ecoul*Binj/(2.0*M_PI*me)) ;
}

// ***************************************************************************************

double a2(double t)
{
   return ((4.0/3.0)*sigmae/(me*c)) * ( (uB(t)/a3)*pow(t,(a1*gaml))
     *(pow(tobs,a3) - pow(t,a3)) + (ucmb/a4)*(pow(tobs,a4) - pow(t,a4)) );
   // factor a2, f(tobs,tinj), in seconds, KDA eq. 7. Tobs is const,
   //    tinj is integrated.
}

// ***************************************************************************************

double rhs(double t)
{      // Hotspot Radius: return r0 + ( F0 * pow( (D(t)/(mperkpc*2.0)), 1.0 ) );
 // r0 = 0.21671769; F0 = 0.020396732; yexp1 = 1.0526278; yexp2 = -0.030605459;
   double texp, t1, t2;
   texp = pow( log(D(t)/mperkpc), 2.0 );
   t1 = exp(yexp2 * texp);
   t2 = pow( (D(t)/mperkpc), yexp1 );
   return r0 + ( F0 * t1 * t2 );
}

// ***************************************************************************************

double Ahs(double t)
{  //  Ahs = M_PI*pow(rhs, 2.0)*mperkpc*mperkpc;    // hotspot area in m^2, Blundell p693
   return M_PI*pow(rhs(t), 2.0)*mperkpc*mperkpc;
}

// ***************************************************************************************

double Bhs(double t)
{
   return sqrt(3.0*mu0*Q0/(c*Ahs(t))) * 1.0e9;    // B field of hotspot in nT,  Blundell p692, eq 11
}

// ***************************************************************************************

double gam0(double t)
{
   double Binj;
   Binj = sqrt(uB(t)*2*mu0);
 //  return sqrt(me*2.0*M_PI*nu/ecoul/(Binj/1.0e9));

   return sqrt(me*2.0*M_PI*nu/ecoul/(Bhs(t)/1.0e9)); // Lorentz factor of a relativistic, radiating electron. BRW eq 3, p687
 // printf("gam0 = %e \n", gam0);
}

// ***************************************************************************************

double gamma(double t)
{
//   return gam0*pow( tobs,(-a1/3.0) )/( pow(t,(-a1/3.0)) - a2(t)*gam0 );
  // return gam0*pow( t,(-a1/3.0) )/( pow(tobs,(-a1/3.0)) - a2(t)*gam0 );
   return gam0(tobs)*pow( t,(-a1/3.0) )/( pow(tobs,(-a1/3.0)) - a2(t)*gam0(tobs) );
   // Lorentz factor as f(tobs,t), KDA eq. 10.
   // t = variable, gam0 = 'gamma', tinj = ti
   // this version solves for Kaiser's gamma_i.
   // Wiita & Osterman think this is the right one
}

// ***************************************************************************************

double gambs(double t)
{
   double nubs;
   nubs = (9.0/4.0)*c7*Bhs(t)/((Bhs(t)*Bhs(t) + Bcmb*Bcmb)*(Bhs(t)*Bhs(t) + Bcmb*Bcmb)*(tbs*tbs));
   return sqrt(nubs * 1.0e6 / nug(t)) * pow( (Bhs(t)*Bhs(t)/(1.0e18*2.0*mu0*uB(t))),
     (-2.0/(3.0*gaml)) );
}

// ***************************************************************************************

double gambf(double t)
{
   double nubf;
   nubf = (9.0/4.0)*c7*Bhs(t)/((Bhs(t)*Bhs(t) + Bcmb*Bcmb)*(Bhs(t)*Bhs(t) + Bcmb*Bcmb)*(tbf*tbf));
   return sqrt(nubf * 1.0e6 / nug(t)) * pow( (Bhs(t)*Bhs(t)/(1.0e18*2.0*mu0*uB(t))),
     (-2.0/(3.0*gaml)) );
}
// Lorentz factors corresponding to nubs and nubf, Blundell eq. 13, p693
// nubs & nubf are converted to Hz, Bhs is converted to T in these eqns.

// ***************************************************************************************

double f(double t)
{
   return pow( (gambs(t)/gambf(t)), pm );  // as determined from Blundell Fig. 11, p694
}

// ***************************************************************************************

double nb(double t)
{
   return (ue(t)/me/c/c) / (  gambs(t)*gambs(t)*log(gambs(t)/gammin) + 
     pow(gambs(t),pm)/(pm - 2.0)*( pow(gambs(t),(2.0 - pm)) - pow(gambf(t),
     (2.0 - pm)) ) + f(t)*pow(gambf(t),3.0)*( 1.0/gambf(t) - 1.0/gammax )  );
   // normalization factor nb, Blundell eq. 16.
}

// ***************************************************************************************

// The functions P1term1, P2term1, P3term1 make up Blundell eq. 21

/*double P1term(double t)
{
   return Pconst*uB(t)*gamma(t)*nb(t)*pow(gambs(t),2.0)
     * pow( pl(t),(-1.0/gaml) )
     * pow( (tobs/t),(-a1*(gaml + 1.0/3.0)) );
  // P1term integrates from tbs to tobs in steps dtinj.
}*/

void P1term1(double t, double y[], double dydx[])
{
dydx[1]=Pconst*uB(t)*gamma(t)*nb(t)*pow(gambs(t),2.0)
     * pow( pl(t),(-1.0/gaml) ) * pow( Ahs(t),((gaml - 1.0)/gaml) ) 
     * pow( (tobs/t),(-a1*(gaml + 1.0/3.0)) );
}

// ***************************************************************************************

/*double P2term(double t)
{
   return uB(t) * pow( gamma(t),(3.0 - pm) ) * nb(t) * pow( gambs(t),pm)
      * pow( pl(t),(-1.0/gaml) )
      * pow( (tobs/t),(-a1*(gaml + 1.0/3.0)) ) * Pconst;
   // P2term integrates from tbf to tbs in steps dtinj.
}*/

void P2term1(double t, double y[], double dydx[])
{
dydx[1]=Pconst*uB(t)* pow( gamma(t), (3.0-pm) ) *nb(t)*pow(gambs(t), pm)
     * pow( pl(t),(-1.0/gaml) ) * pow( Ahs(t),((gaml - 1.0)/gaml) ) 
     * pow( (tobs/t),(-a1*(gaml + 1.0/3.0)) );
}

// ***************************************************************************************

/*double P3term(double t)
{
   return uB(t) * f(t) * pow(gambf(t),(3.0))
      * pow( pl(t),(-1.0/gaml) )
      * nb(t) * Pconst * pow( (tobs/t),(-a1*(gaml + 1.0/3.0)) );
   // P3term integrates from tmin to tbf in steps dtinj.
}*/
