/*  M. Angela Osterman
    April 18, 2003
   This program will follow Blundell's redshift distribution model and use
    it to generate a sample of sources with quasi-random tage. Then, BRW's
    Q0 distribution is added to get the full tobs(tage)-Q0-zobs dist.  

    Paramita Barai
    April 27, 2004

2004 May 24: start at z=10, modify the range of counts in arrsiz & sourcesum 
                            (as was missing the last before)
May 28: Corrected obtaining chi_src from ii [replaced (0.33333) by (1.0/3.0)]
	[Now get both low & high tage, according to the random # generated.]
Changed the way random # was generated & assigned to chi1 & chi2
Removed (2.0*M_PI)^3 factor from bottom of dV [comoving volume]
Given check condition to see if BREAK occured or not ..

Aug 27: Steepening Power Law Q_0 distribution, making Q_0^(-3.0), 
i.e. make Power Law Index, x = 3.0 = Q0index (here in code)

2005
Mar 25 : Decrease T_maxage to 50-400 Myr & see what happens

/nfs/morgan3/wiita/Papers08/Barai/flat_tqzdV/x3p0/MaxAge150Myr
brw_tqz.c	*/


#include <math.h>     // add -lm to compiler command.
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include "nrutil.h"   // make sure this header file is in this directory.
#include "nr.h"

// the following are required for the Numerical Recipes functions.
#define EPS 1.0e-6
#define JMAX 20
#define JMAXP (JMAX+1)
#define Kconst 5
#define NRANSI
#define FUNC(x) ((*func)(x))
#define NR_END 1
#define FREE_ARG char*  

// write pi as M_PI
#define c 2.9979e8             // speed of light in m/s.
#define H0 71.00               // Hubble constant in km/sec/Mpc. NEW COSMOLOGY
#define omegaM 0.3           // matter term
#define omegaL 0.7            // vacuum term
#define secpermyr 3.15576e13   // the number of seconds in a Myr.
#define kmperpc 3.086e13       // the number of km in a pc.

double tzfactor, omegtot;

int kmax, kount;
double *xp, **yp, dxsav;
double eps, h1, hmin, k1, k2, *fgidist;

int main()
{

void chifunc1(double , double [], double []);
void odeint(double ystart[], int nvar, double x1, double x2, double eps, double h1,
        double hmin, int *nok, int *nbad,
        void (*derivs)(double, double [], double []),
        void (*rkqs)(double [], double [], int, double *, double, double, double [],
        double *, double *, void (*)(double, double [], double [])));
double tofzwein(),zoftwein(),Rscal(), chifunc();
double maxbeamlife,z1,delz1,sampsize,dtbir,tnow,zbir0,tbir0;
double dzdt,zbirprob,normfactor,probsum,jj, ii;
double dtinj, init_tinj, totint, tottime, chival_int;
double toff,zoff,Rbir,Roff,dV,comovdV;
double tval,chival,tmid,tobs;
double powconst,Q0min,Q0max,Q0index,Q0val;
int total42,total41,total40,total39, npieces ;
// unsigned int i,j,k,l,m,n, nn;    // for loop indexes.
int i,j,k,l,m,n, nn, ni, nok, nbad;    // for loop indexes.
long arrsiz, sourcesum;
double chi2r, chi1r, chi_srcr;

double *zbir = (double *)malloc(20000*sizeof(double));
double *tbir = (double *)malloc(20000*sizeof(double));
   // initializes zbir & tbir array to have room for the number of populations
   //    that can be hatched between an initial zbir and tnow.
double *tbirdist = (double *)malloc(20000*sizeof(double));
double *tbirdistrnd = (double *)malloc(20000*sizeof(double));
double *chi1 = (double *)malloc(20000*sizeof(double));
double *chi2 = (double *)malloc(20000*sizeof(double));
double *chi_src = (double *)malloc(10000000*sizeof(double));
double *tbir_src = (double *)malloc(10000000*sizeof(double));
double *zbir_src = (double *)malloc(10000000*sizeof(double));
double *zobs = (double *)malloc(10000000*sizeof(double));
double *tageATobs = (double *)malloc(10000000*sizeof(double));

time_t t1,t2;     // declares a time type thing.
FILE *stream1, *stream2, *stream3, *fopen(), *stream4, *stream5;

xp = dvector(1, 200);
yp = dmatrix(1, 10, 1, 200);
fgidist = dvector(1, 1);

arrsiz = 20000; 
// Constants and their assumed values
omegtot = omegaM + omegaL;   // total density parameter;
tzfactor = 2.0/3.0 * sqrt(1.0 + omegaM/omegaL) * (1.0/H0) * 1.0e6 * kmperpc;
          // factor from Weinberg eq. 5.4, gives resulting time in s
maxbeamlife = 150.0*secpermyr;        // (MaxAge150Myr)
  // max time for source beams to be on (Blundell)

sampsize = 0.6e1;   // number of sources in sample.
z1 = 2.20;       // z1=2.2 and delz1=0.6 are Blundell's values used in eq 24.
delz1 = 0.60;

/* Redshifts at which source was born, user chosen vals. They are set to begin
    at zbir = 10 and subsequent zbir vals will come in intervals (dt) of 
    0.1 million years, this creates 128193 redshift values.    */
dtbir = 1.0*secpermyr;
tnow = tofzwein(1.0e9); // this is the current age of the universe, found
                        //  by putting a z ~ infinity into Weinberg's eq 54.
zbir0 = 10.0;    // this is the earliest zbir, following BRW.
tbir0 = tnow - tofzwein(zbir0);
printf("tnow = %e , \t tbir0 = %e \n", tnow, tbir0);

zbir[0] = zbir0;     // this is the earliest zbir, user chosen.         
tbir[0] = tbir0;
for (i = 1 ; i < arrsiz ; i++)
  {
     tbir[i] = tbir[i-1] + dtbir;
     zbir[i] = zoftwein(tnow - tbir[i]);
     if ( zbir[i] < 0 )
	{ 
	printf("z coming < 0, breaking loop, i = %d ", i);
        arrsiz = i-1;
	break; 
	}
     // printf("%e \t %e \n",zbir[i],tbir[i]/secpermyr);
  }
printf("Initial zbir and tbir calculations complete. \n");


/* Assume that Blundell's zobs distribution (eq 24) also applies to zbir. Then
    the number of sources in the sample corresponding to a given tbir is
    calculated from multiplying the size of the sample by rho(zobs) (eq 24). */
normfactor = 1.0/(sqrt(2.0*M_PI)*delz1);    // normalization factor, following 
                                            //    Gaussian form.
// printf("%e \n",normfactor);
tbirdist[0] = 0.0;     tbirdistrnd[0] = 0.0;
   // this is a fair assumption for z = 10.0.

// open file for saving comoving dV and z to compare w/BRW.
stream1 = fopen("ZcomdV.dat","w");
if ( (stream1 = fopen("ZcomdV.dat","w")) == NULL)
    {   printf("Can't open %s \n","ZcomdV.dat");   exit(1);    }
// This loop will calculate comoving dV, to be plotted vs. z in IDL. This
//   depends only on tbir and zbir, not the sample size.
for (m = 0 ; m <= arrsiz ; m++)
  {

    // toff is the time corresponding to beams switching off in sec.
    toff = tbir[m] + maxbeamlife;

    // zoff is the redshift at which the source switches off. 
    zoff = zoftwein(tnow - toff);

    // Calculate the scale factor R at tbir and toff.
    // Note that it R/R0 is what is actually calculated, this ratio
    //   should be 0 at the Big Bang and 1 and present time.
    Rbir = Rscal(tbir[m]);
    Roff = Rscal(toff);

    // The values of chi1 and chi2 are found by integrating Blundell eq
    //   35 and 34, respectively  
   // The next block of code integrates chifunc by odeint to give chi1[m]
   k1 = toff;
   k2 = tnow;
   eps = 1.0e-5;
   h1 = 1.0e-2;
   hmin = 0.0;
   fgidist[1] = 0.0;
   odeint(fgidist, 1, k1, k2, eps, h1, hmin, &nok, &nbad, chifunc1, rkqs);
   chi1[m] = fgidist[1];

 // The next block of code integrates chifunc by odeint to give chi2[m]
   k1 = tbir[m];
   k2 = tnow;
   eps = 1.0e-5;
   h1 = 1.0e-2;
   hmin = 0.0;
   fgidist[1] = 0.0;
   odeint(fgidist, 1, k1, k2, eps, h1, hmin, &nok, &nbad, chifunc1, rkqs);
   chi2[m] = fgidist[1];

    // Calculate deltaV = dV in Gpc^3 (at redshift zero). Following
    //    Blundell eq31 for the flat universe (k = 0).
    dV = 2.0*M_PI*(2.0/3.0)*pow(Rbir,3)*( pow(chi2[m],3) - pow(chi1[m],3) ) / 
          pow(  ( 1.0e3 * kmperpc * 1.0e9 ),3  );
    comovdV = dV*pow( (1.0 + zbir[m]),3 );
    dzdt = zbir[m-1] - zbir[m];
    zbirprob = exp(  -(1.0/2.0) * pow( ((zbir[m] - z1)/delz1),2.0 )  );
         // not normalized.
    // tbirdist[m] = comovdV*sampsize*normfactor*zbirprob*dzdt;
     tbirdist[m] = comovdV*sampsize*normfactor*zbirprob;
     tbirdistrnd[m] = floor(tbirdist[m]);  // DO NOT USE CEIL!!!
   //   sourcesum = sourcesum + floor(tbirdist[m]);
    fprintf(stream1," %e \t %e \n",zbir[m],comovdV);
    printf("on run %i, zbir=%e, tbirdist=%e, dzdt=%e, zbirprob=%e\n",m, zbir[m], tbirdist[m], dzdt, zbirprob);

  }
// fclose(stream1);
printf("The comoving dV calculations are complete. \n");

/* When assigning sources to redshifts, just counting sources where the z
     probablity is more than 0.5 will exclude the rare but realistic
     occurences of sources and very high and very low redshifts. To
     counteract this, a few redshifts of low probability will be chosen by
     adding a counter, probsum, that will allow a low-prob z to be chosen
     every time that probsum becomes greater than one. After a z is chosen,
     probsum is reset to zero.     */

stream5 = fopen("tbirdistrnd_z.dat","w");
if ( (stream5 = fopen("tbirdistrnd_z.dat","w")) == NULL)
    {   printf("Can't open %s \n","tbirdistrnd_z.dat");   exit(1);    }

probsum = 0.0;       // this counter will keep count of the probability sum.
sourcesum = 0.0;     // this counter will count up the number of sources.
//for (k = 1 ; k <= arrsiz ; k++)
for (k = 0 ; k <= arrsiz ; k++)
  {
    if ( tbirdist[k] < 1.0 ) probsum += tbirdist[k];
    if ( probsum >= 1.0 ) tbirdist[k] = 1.0;
    if ( probsum >= 1.0 ) tbirdistrnd[k] = floor(tbirdist[k]);

    //printf("%e \t %e \t %e \t %e \n", tbir[k], tbirdist[k], tbirdistrnd[k], zbir[k]);
    fprintf(stream5, "%e \t %e \t %e \t %e \n", tbir[k], tbirdist[k], tbirdistrnd[k], zbir[k]);

    sourcesum += floor(tbirdist[k]);
    if ( probsum >= 1.0 ) probsum = 0.0;
    printf("on run %i, zbir = %e, tbirdist = %e, sourcesum = %d \n",k, zbir[k], tbirdist[k], sourcesum);
  }
printf("total = %d, total should = %e. \n",sourcesum, sampsize);
// printf("Finished distribution of tbir values throughout sample. \n");                                                                                                             
printf("\n Sourcesum = %d \n", sourcesum);
fprintf(stream1, "\n Sourcesum = %d \n", sourcesum);
fclose(stream1);
fclose(stream5); 

/* The time, tobs, when a source will intercept our light cone is determined by
    its radial distance from us at the cosmic time corresponding to zbir within
    that radial shell. Following spatial homogeneity, the sources will be
    randomly distributed through the volume shell dV. Thus, the length of time
    that elapses after a given source is born before it intercepts our light
    cone (tage) is randomly drawn from 0 < tage < 500 million years, but not
    with a uniform distribution.
  To obtain the relavent tobs and tage values: (1) randomly allocate sources to
    equal volume shells based on the tbir value. Each tbir value has a
    corresponding chi1, chi2, and dV. (2) Solve for the chi of the volume
    subshell by inverting Blundell eq 31.    */
// The next set of calculations requires a random number:

stream4 = fopen("tzchi.dat","w");
if ( (stream4 = fopen("tzchi.dat","w")) == NULL)
    {   printf("Can't open %s \n","tzchi.dat");   exit(1);    }

(void) time(&t1);
srand48((long) t1);   // uses system time in seconds to set the seed.
n = 0;    // initialize index above three arrays.
for (i = 0 ; i <= arrsiz ; i++)
  {
    if (tbirdistrnd[i] > 0.0)  {
     // Assume 50 delV subshells in which to randomly allocate each source.
    for (m = 1 ; m <= tbirdistrnd[i] ; m++)
      {
   // jj = 50.0 - 50.0*drand48();
   //chi_src[n] = pow(  ( (jj*pow(chi2[i],3) + (50.0-jj)*pow(chi1[i],3))*1.0/50.0 ),0.33333  );
	ii = drand48();
	chi2r = chi2[i]/1.0e26;
        chi1r = chi1[i]/1.0e26;
	chi_srcr = pow(  ( ii*pow(chi2r,3) + (1.0-ii)*pow(chi1r,3)  ), (1.0/3.0)  );
        chi_src[n] = chi_srcr*1.0e26;
        tbir_src[n] = tbir[i];
        zbir_src[n] = zbir[i];
        //printf("%e \t %e \t %e \t %e \n", tbir_src[n]/secpermyr, zbir_src[n], chi_src[n],jj);
	fprintf(stream4, "%e \t %e \t %e \t %e \n", tbir_src[n]/secpermyr, zbir_src[n], chi_src[n], ii);
        n = n + 1;
      }
                          }
  }
printf("The chi_src calculations are complete.\n");
fprintf(stream4, "The chi_src calculations are complete.\n, n = %d\n", n); 
fclose(stream4);


/* Once the chi of each source is calculated, the values of tobs and tage can
       be calculated from Blundell eq 36 and 37, respectively.    */

stream3 = fopen("check.dat","w");
if ( (stream3 = fopen("check.dat","w")) == NULL)
     {   printf("Can't open %s \n","check.dat");   exit(1);   }

stream4 = fopen("NObreak.dat","w");
if ( (stream4 = fopen("NObreak.dat","w")) == NULL)
     {   printf("Can't open %s \n","NObreak.dat");   exit(1);   }

n = 5000;       // divide tbir to toff into n shells

for (i = 0 ; i <= sourcesum ; i++)
  {
    tval = tbir_src[i] + maxbeamlife/2.0;
    tmid = maxbeamlife/2.0;
    for (m = 0 ; m <= n ; m++)
    {
      k1 = tval;
      k2 = tnow;
      eps = 1.0e-5;
      h1 = 1.0e-2;
      hmin = 0.0;
      fgidist[1] = 0.0;
      odeint(fgidist, 1, k1, k2, eps, h1, hmin, &nok, &nbad, chifunc1, rkqs);
      chival = fgidist[1];
//      printf("chival successfully found, chival = %e, \t m = %d  \n", chival, m);
      tmid = tmid/2.0;
      if ( fabs(chi_src[i]-chival) < 1.e+18 ) break;
      if ( chi_src[i] > chival ) tval -= tmid;
      if ( chi_src[i] < chival ) tval += tmid;
      if (m == n) 
      {
fprintf(stream4, " NO BREAK: chi_src[%d] = %e\t tval (Myr) = %e\n", i, chi_src[i], tval/secpermyr);
      }	 
    }
      zobs[i] = zoftwein(tnow - tval);
      tageATobs[i] = tval - tbir_src[i];
      printf("%e \t %e \t %e\t%e\t%e \n",zbir_src[i],zobs[i],(tval - tbir_src[i])/secpermyr, tval/secpermyr, tbir_src[i]/secpermyr);

    fprintf(stream3,"%d \t %e \t %e \t %e\t%e \n",i, zbir_src[i],zobs[i],(tageATobs[i])/secpermyr, tbir_src[i]/secpermyr);
  } 
printf("The zobs at tage@obs calculations are complete.\n");
fclose(stream3);
fclose(stream4);

/* New random numbers will correspond to probability of choosing a source of
    power up to Q for a given Q. IT IS NECESSARY TO HAVE DIFFERENT RANDOM
    NUMBERS FOR THE CHI_SRC AND Q0 DISTRIBUTIONS, SCIENTIFICALLY.
  The CUMULATIVE probablity of choosing a source with power up to Q is:
  P(Q) = INT_Q_1^Q (K Q^-2.6 dQ) / [ INT_Q_1^Q_2 (K Q^-2.6 dQ) ]
  If X is chosen randomly from 0 to 1 then we should be able to equate X with
     P(Q). Clearly, the K's cancel as do the 1/-1.6 from each integral;
  P(Q) = X = [Q_1^-1.6 - Q^-1.6]/[Q_1^-1.6 - Q_2^-1.6]
  Then we solve for the appropriate random value of that will reproduce the
     desired distribution (Blundell eq 38):
  Q = [ Q_1^-1.6 - X (Q_1^-1.6 - Q_2^-1.6) ]^-0.625     */

// open file to save zobs, tage(tobs), Q0 data.
stream2 = fopen("tbirztageQ.dat","w");
if ( (stream2 = fopen("tbirztageQ.dat","w")) == NULL)
     {   printf("Can't open %s \n","tbirztageQ.dat");   exit(1);   }

total42 = 0;   total41 = 0;   total40 = 0;   total39 = 0;
(void) time(&t2);
srand48((long) t2);   // uses system time in seconds to set the seed.
Q0min = 5.0e37;   // Min possible source power (W), Blundell = 5e37, p699
Q0max = 5.0e42;   // Max   "        "     "     " , BRW = 5e42, p699
//Q0index = 2.60;   // BRW x = 2.6; power law index of eqn 38, p699
Q0index = 3.0;   // Changing Power Law index

//printf("\n B4 last loop, sourcesum = %d \n", sourcesum);
for (i = 0 ; i <= sourcesum ; i++)
  {
    Q0val=pow( (pow(Q0min,(-Q0index+1.0)) - drand48()*(pow(Q0min,(-Q0index+1.0))
               - pow(Q0max,(-Q0index+1.0)))),(1.0/(-Q0index+1.0)) );
//   printf(" Q0val = %e \n", Q0val); 
   if ( log10(Q0val) >= 42.0 ) total42 += 1;
    if (( log10(Q0val) < 42.0 ) && ( log10(Q0val) >= 41.0 )) total41 += 1;
    if (( log10(Q0val) < 41.0 ) && ( log10(Q0val) >= 40.0 )) total40 += 1;
    if (( log10(Q0val) < 40.0 ) && ( log10(Q0val) >= 39.0 )) total39 += 1;
    fprintf(stream2,"%e \t %e \t %e \t %e \n",tbir_src[i]/secpermyr,
        zobs[i], tageATobs[i]/secpermyr,Q0val);
  }

fclose(stream2);
fclose(stream3);
printf("The Q0 distribution is complete.\n");
printf("There are %i sources with logP >= 42.\n",total42);   // 1
printf("There are %i sources with logP >= 41.\n",total41);   // 12
printf("There are %i sources with logP >= 40.\n",total40);   // few 100
printf("There are %i sources with logP >= 39.\n",total39);   // ~12500
printf("\n Sampsize = %d \n", sourcesum);

free(zbir);   free(tbir);    free(tbirdist);   free(tbirdistrnd);
free(chi1);   free(chi2);    free(chi_src);   free(zbir_src);   free(tbir_src);
free(zobs);   free(tageATobs);

return 0;
}

// ****************************************************************************

double tofzwein(double zval)
{
/* Weinberg eq 5.4, two factors divide (#km per Mpc) by km so that
      times are calculated in seconds. 
   Note that this calculates the current age of a source that was born
     at a given redshift.     */
   return tzfactor*(  asinh(sqrt(omegaL/omegaM))
         - asinh(sqrt(omegaL/omegaM)*pow( (1.0 + zval),(-3.0/2.0) ))  );
}


double zoftwein(double tval)
{
// This is Weinberg eq 5.4 solved for z.
   return pow(  ( sinh(asinh(sqrt(omegaL/omegaM)) - tval/tzfactor)
       / sqrt(omegaL/omegaM) ),(-2.0/3.0) ) - 1.0;
}


double Rscal(double tval)
{
// The scale factor a = R/R0 is calc by inverting Peacock eq. 3.63 for t(a).
// This function returns a unitless quantity!
// CHECK sinh,abs
   return pow(  ( sinh((3.0/2.0)*(H0/1.0e6/kmperpc)*tval*sqrt(fabs(omegaM-1.0)))
           /sqrt(fabs(omegaM - 1.0)/omegaM) ),(2.0/3.0)  );
}

// ***************************************************************************

//double chifunc(double tval)
void chifunc1(double tval, double y[], double dydx[])
{
// The scaling factors should produce units of Gpc^3. for dV.
   dydx[1] = c/Rscal(tval);
}

// *************************************************************************

double chifunc(double tval)
{
// The scaling factors should produce units of Gpc^3. for dV.
   return c/Rscal(tval);
}

// *****************************************************************************

